/* controlling motor from IR */

/* 
pins list

A0 =  Current sense motor 1
A1 = Current sense motor 2
A2 = Current sense motor 3
A3 = Current sense motor 4
A4 = unused
A5 = unused

D0 = unused
D1 = unused
D2 = quadrature motor 1 (left hand front)
D3 = quadrature motor 2 (right hand front)
D4 = **** IR Receiver

D5 = motor drive motor 1
D6 = motor drive motor 2
D7 = motor direction motor 1
D8 = motor direction motor 2

D9 = motor drive motor 3
D10 = motor drive motor 4
D11 = motor direction motor 3
D12 = motor direction motor 4

Motor 1 = A0, D5, D7
Motor 2 = A1, D6, D8
Motor 3 = A2, D9, D11
Motor 4 = A3, D10, D12

*/


//****************************** Motor set up

const int A = 0;
const int S = 1;
const int D = 2;

int motor1[] = {A0, 5, 7};
int motor2[] = {A1, 6, 8};
int motor3[] = {A2, 9, 11};
int motor4[] = {A3, 10, 12};

/*  ************ Motor Calibration 
Quadrature readings at speed = 40, with tracks fitted angle =??
*/

const float calMotor1 = 8180.25; //(8281 + 8210 + 8159 + 8071)/4.0;
const float calMotor2 = 7704.5; // (7829 + 7785 + 7597 + 7607)/4.0;
const float calMotor3 = 7669 ;//(7669 + 7694 + 7743 + 7570)/4.0;
const float calMotor4 = 8181.75 ;//(8255 + 8216 + 8142 + 8114)/4.0;


// ************************** Infra Red
#include <IRremote.h>           // IR remote control library

const int irReceivePin = 4;     // pin connected to IR detector output
IRrecv irrecv(irReceivePin);    // create the IR library
decode_results results;         // IR data goes here
long counter = 0l;

const int numberOfKeys = 20;     

const String keyNames[numberOfKeys] = {"OSD", "Date/time", "Photo Shoot", "Start/Stop", "Zoom T", "Zoom W", "Rew",
"Play", "FF","Still Adv-", "Pause", "Still Adv+", "Stop", "Var Search", "Up", "Menu", "left","Enter",
"Right","Down"};

const int K_START_STOP = 3;
const float AMP_SCALING = 5.0/1023;

const unsigned long long panasonicCodes[numberOfKeys] = {70386160036596ull, 70386160035576ull,
70386160002680ull, 70386159989549ull, 70386159994456ull, 70386160027352ull, 
70386159992926ull, 70386159997006ull, 70386160025822ull, 70386160034046ull,
70386160001150ull, 70386160038126ull, 70386159976478ull, 70386168365471ull,
70386168368021ull, 70386160003700ull, 70386168417109ull, 70386159983108ull,
70386168384469ull, 70386168400661ull};


int inPins[] = {A0,A1,A2,A3, 2, 3, 4};
int outPins[]={5,6,7,8,9,10,11,12};

const int SPEED_STOPPED = 0;
int speed = 200;
int currentTime= 0;
int i ;

float amps;

boolean dir = LOW;
long quadrature0Pulses = 0l;
long quadrature1Pulses = 0l;
long quadrature0Start = 0l;  // for timing quadrature pulses per unit time.
long quadrature1Start = 0l;
String message;

int STOPPED = false;
int STARTED  = true;
boolean motorState = STOPPED;
long time_last_read = 0;
// debounce time needs to be different for different sorts of keypress?
// how to deal with repeat of key??


void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
    irrecv.enableIRIn();              // Start the IR receiver

  Serial.println("Out pins");
  for ( i = 0; i < 8 ; i++){
    pinMode (outPins[i], OUTPUT);
    Serial.println(outPins[i]);
    }
  
  Serial.println("in pins");
  for (i = 0; i < 7 ; i++){
    pinMode(inPins[i], INPUT); 
        Serial.println(inPins[i]);
      }

  attachInterrupt(0,quadrature0, RISING);
  attachInterrupt(1,quadrature1, RISING);

}

void loop(){
  long key;
  counter +=1;
  if (irrecv.decode(&results))
  {

    // here if data is received
    irrecv.resume();
    key = convertCodeToKey(results.value);
    if(key >= 0) {
      Serial.print(counter);
      Serial.print(": Got key ");
      Serial.print(key);
      Serial.print(" - ");
      Serial.print(keyNames[key]);
      Serial.print("  @ ");
      Serial.println(time_last_read);
    //  brightness = map(key, 0,numberOfKeys-1, 0, 255);
    //  analogWrite(ledPin, brightness);
    }
    
    
    if (key == K_START_STOP  && millis() > time_last_read+500){
      // if started, stop, if stopped, start (all motors)
      if (motorState){
          dumpCurrent(motor1);
          dumpCurrent(motor2);
          dumpCurrent(motor3);
          dumpCurrent(motor4);
        
          runMotor(motor1, dir, SPEED_STOPPED) ;
          runMotor(motor2,dir,SPEED_STOPPED);
          runMotor(motor3,dir,SPEED_STOPPED);
          runMotor(motor4,dir,SPEED_STOPPED);
          
          processQuadrature();
      }
      else{
          runMotor(motor1, dir, speed) ;
          runMotor(motor2,dir,speed);
          runMotor(motor3,dir,speed);
          runMotor(motor4,dir,speed);
      }
      // toggle motor state
      motorState = !motorState;
      
    }
    time_last_read = millis();
  }
}

void loop0() {
  // put your main code here, to run repeatedly:  
  currentTime = millis()/1000; // time in seconds
  
  Serial.println("Testing motor1");
  runMotor(motor1, dir, speed) ;
  runMotor(motor2,dir,speed);
  runMotor(motor3,dir,speed);
  runMotor(motor4,dir,speed);
  
  delay(5000);
  
  Serial.println("Current in motors 1-4");
  dumpCurrent(motor1);
  dumpCurrent(motor2);
  dumpCurrent(motor3);
  dumpCurrent(motor4);
  
  Serial.println("quadrature");
  Serial.println(quadrature0Pulses);
  Serial.println(quadrature1Pulses);
  
  
  runMotor(motor1,dir,0);
  runMotor(motor2,dir,0);
  runMotor(motor3,dir,0);
  runMotor(motor4,dir,0);
/*  
  dir = getDirection(currentTime);
  speed = getSpeed(currentTime);
  
  digitalWrite(directionPin,dir );
  analogWrite(pwmPin, speed); 
  amps = analogRead(currentPin);
  */
//  amps = analogRead(motor1[A]);
//  if (millis()%1000 ==0){
//  Serial.println(amps);
//  Serial.println(quadrature0Pulses, quadrature1Pulses);}
  
  delay(5000);

}



void printll(unsigned long long n)
{
 char output[30];
 byte d=0;
 while (n > 0)
 {
   output[d++] = '0' + (char)(n % 10ULL);
   n = n / 10ULL;
 }
 while (d-- > 0) //reverse digits and print
 {
   Serial.print(output[d]);
 }
}
    


/*
 * converts a remote protocol code to a logical key code (or -1 if no digit
received)
 */
int convertCodeToKey(long long code)
{
  for( int i=0; i < numberOfKeys; i++)
  {
    if( code == panasonicCodes[i])
    {
      return i; // found the key so return it
    }
  }
  return -1;
}


void dumpCurrent(int motor[]){
  amps = analogRead(motor[A]);
  
  Serial.print(amps);
  Serial.print(" Analog reading = ");
  Serial.print(amps*AMP_SCALING);
  Serial.println(" A");
}

void runMotor(int motor[], boolean d, int s){
  digitalWrite(motor[D],d);
  analogWrite(motor[S],s);
  Serial.print("Running motor, direction = ");
  Serial.print(d);
  Serial.print(" Setting speed to: ");
  Serial.println(s);

}


void quadrature0(){
  if (quadrature0Start == 0){
    quadrature0Start = millis();
    return;
  }
  quadrature0Pulses += 1;
}

void quadrature1(){
    if (quadrature1Start == 0){
      quadrature1Start = millis();
      return;
    }
  quadrature1Pulses +=1;
}

void processQuadrature(){
  long now = millis();
  
  Serial.print("Quadrature 0  = ");
  Serial.print(float(quadrature0Pulses*1000)/(now-quadrature0Start) );
  Serial.print(" pulses per second  over ");
  Serial.print(now-quadrature0Start);
  Serial.println(" milliseconds");
  
  Serial.print("Quadrature 1  = ");
  Serial.print(float(quadrature1Pulses*1000)/(now-quadrature1Start) );
  Serial.print(" pulses per second over ");
  Serial.print(now-quadrature1Start);
  Serial.println(" milliseconds");
  
  quadrature0Start = 0l;
  quadrature1Start = 0l;

  quadrature0Pulses = 0l;
  quadrature1Pulses = 0l;
  
  Serial.print("Resetting Quadrature: 0: ");
  Serial.print(quadrature0Start);
  Serial.print(" 1: ");
  Serial.println(quadrature1Start);
  
}


boolean getDirection(int t){
  int t1 = t/5;
  switch (t1%4) {
  case 0:
    return HIGH;
    break;
  case 1: 
    return HIGH;
    break;
  case 2:
    return LOW;
    break;
   case 3:
    return LOW;
    break;
  }
}

int getSpeed(int t){
  int t1 = t/5;
  switch (t1%4) {
  case 0:
    return 40;
    break;
  case 1:
    return 0;
    break;
  case 2:
    return 40;
    break;
  case 3:
    return 0;
    break;
  }
  }
